@@ -1,16 +1,22 @@
+libDir = node.path.join(node.path.dirname(__filename), "../lib");
+node.libraryPaths.unshift(libDir);
+
+node.mixin(require("/utils.js"));
+http = require("/http.js");
+
 fixed = ""
 for (var i = 0; i < 20*1024; i++) {
   fixed += "C";
 }
 stored = {};
-node.http.createServer(function (req, res) {
+http.createServer(function (req, res) {
   var commands = req.uri.path.split("/");
   var command = commands[1];
   var body = "";
   var arg = commands[2];
   var status = 200;
 
-  //p(req.headers);
+  p(req.uri.params);
 
   if (command == "bytes") {
     var n = parseInt(arg, 10)
