@@ -155,7 +155,7 @@ class Serializer: public ObjectVisitor {
   friend class ReferenceUpdater;
 
   virtual void VisitPointers(Object** start, Object** end);
-
+  virtual void VisitCodeTarget(RelocInfo* rinfo);
   bool IsVisited(HeapObject* obj);
 
   Address GetSavedAddress(HeapObject* obj);
@@ -197,7 +197,7 @@ class Serializer: public ObjectVisitor {
 
   int flags_end_;  // The position right after the flags.
 
-  // An array of per-space SimulatedHeapSpacees used as memory allocators.
+  // An array of per-space SimulatedHeapSpaces used as memory allocators.
   SimulatedHeapSpace* allocator_[LAST_SPACE+1];
   // A list of global handles at serialization time.
   List<Object**> global_handles_;
@@ -289,6 +289,7 @@ class Deserializer: public ObjectVisitor {
 
  private:
   virtual void VisitPointers(Object** start, Object** end);
+  virtual void VisitCodeTarget(RelocInfo* rinfo);
   virtual void VisitExternalReferences(Address* start, Address* end);
   virtual void VisitRuntimeEntry(RelocInfo* rinfo);
 
