@@ -160,18 +160,6 @@ void Debug::GenerateReturnDebugBreak(MacroAssembler* masm) {
 }
 
 
-void Debug::GenerateReturnDebugBreakEntry(MacroAssembler* masm) {
-  // OK to clobber rbx as we are returning from a JS function through the code
-  // generated by CodeGenerator::GenerateReturnSequence()
-  ExternalReference debug_break_return =
-      ExternalReference(Debug_Address::DebugBreakReturn());
-  __ movq(rbx, debug_break_return);
-  __ movq(rbx, Operand(rbx, 0));
-  __ addq(rbx, Immediate(Code::kHeaderSize - kHeapObjectTag));
-  __ jmp(rbx);
-}
-
-
 void Debug::GenerateStoreICDebugBreak(MacroAssembler* masm) {
   // REgister state for IC store call (from ic-x64.cc).
   // ----------- S t a t e -------------
@@ -207,7 +195,7 @@ bool BreakLocationIterator::IsDebugBreakAtReturn()  {
 
 void BreakLocationIterator::SetDebugBreakAtReturn()  {
   ASSERT(Debug::kX64JSReturnSequenceLength >= Debug::kX64CallInstructionLength);
-  rinfo()->PatchCodeWithCall(Debug::debug_break_return_entry()->entry(),
+  rinfo()->PatchCodeWithCall(Debug::debug_break_return()->entry(),
       Debug::kX64JSReturnSequenceLength - Debug::kX64CallInstructionLength);
 }
 
