@@ -1,6 +1,5 @@
-#include "events.h"
-#include <ev.h>
-#include <v8.h>
+// Copyright 2009 Ryan Dahl <ry@tinyclouds.org>
+#include <events.h>
 
 #include <assert.h>
 #include <stdlib.h>
@@ -13,36 +12,44 @@
 #include <arpa/inet.h> /* inet_ntop */
 #include <netinet/in.h> /* sockaddr_in, sockaddr_in6 */
 
+#include <node.h>
+#include <ev.h>
+#include <v8.h>
+
+namespace node {
+
 #ifndef RAMP
 # define RAMP(x) ((x) > 0 ? (x) : 0)
 #endif
 
 using namespace v8;
-using namespace node;
 
 Persistent<FunctionTemplate> EventEmitter::constructor_template;
 
 /* Poor Man's coroutines */
 static Promise *coroutine_top;
+static int coroutine_stack_size;
 
-void
-EventEmitter::Initialize (Local<FunctionTemplate> ctemplate)
-{
+void EventEmitter::Initialize(Local<FunctionTemplate> ctemplate) {
   HandleScope scope;
 
   constructor_template = Persistent<FunctionTemplate>::New(ctemplate);
 
   Local<FunctionTemplate> __emit = FunctionTemplate::New(Emit);
-  constructor_template->PrototypeTemplate()->Set(String::NewSymbol("emit"), __emit);
+  constructor_template->PrototypeTemplate()->Set(String::NewSymbol("emit"),
+      __emit);
+  constructor_template->SetClassName(String::NewSymbol("EventEmitter"));
 
   // All other prototype methods are defined in events.js
 
   coroutine_top = NULL;
+  coroutine_stack_size = 0;
 }
 
-static bool
-ReallyEmit (Handle<Object> self, Handle<String> event, int argc, Handle<Value> argv[])
-{
+static bool ReallyEmit(Handle<Object> self,
+                       Handle<String> event,
+                       int argc,
+                       Handle<Value> argv[]) {
   HandleScope scope;
 
   Local<Value> events_v = self->Get(String::NewSymbol("_events"));
@@ -73,21 +80,21 @@ ReallyEmit (Handle<Object> self, Handle<String> event, int argc, Handle<Value> a
   return true;
 }
 
-Handle<Value>
-EventEmitter::Emit (const Arguments& args)
-{
+Handle<Value> EventEmitter::Emit(const Arguments& args) {
   HandleScope scope;
-  Local<String> event = args[0]->ToString();
-  int argc = 0;
-  Local<Array> emit_args;
-  if (args[1]->IsArray()) {
-    emit_args = Local<Array>::Cast(args[1]);
-    argc = emit_args->Length();
+
+  if (args.Length() == 0) {
+    return ThrowException(Exception::TypeError(
+          String::New("Must give event name.")));
   }
+
+  Local<String> event = args[0]->ToString();
+
+  int argc = args.Length() - 1;
   Local<Value> argv[argc];
 
   for (int i = 0; i < argc; i++) {
-    argv[i] = emit_args->Get(Integer::New(i));
+    argv[i] = args[i+1];
   }
 
   bool r = ReallyEmit(args.Holder(), event, argc, argv);
@@ -95,9 +102,7 @@ EventEmitter::Emit (const Arguments& args)
   return scope.Close(r ? True() : False());
 }
 
-bool
-EventEmitter::Emit (const char *event_s, int argc, Handle<Value> argv[])
-{
+bool EventEmitter::Emit(const char *event_s, int argc, Handle<Value> argv[]) {
   HandleScope scope;
   Local<String> event = String::NewSymbol(event_s);
   return ReallyEmit(handle_, event, argc, argv);
@@ -105,15 +110,14 @@ EventEmitter::Emit (const char *event_s, int argc, Handle<Value> argv[])
 
 Persistent<FunctionTemplate> Promise::constructor_template;
 
-void
-Promise::Initialize (v8::Handle<v8::Object> target)
-{
+void Promise::Initialize(v8::Handle<v8::Object> target) {
   HandleScope scope;
 
   Local<FunctionTemplate> t = FunctionTemplate::New(New);
   constructor_template = Persistent<FunctionTemplate>::New(t);
   constructor_template->Inherit(EventEmitter::constructor_template);
   constructor_template->InstanceTemplate()->SetInternalFieldCount(1);
+  constructor_template->SetClassName(String::NewSymbol("Promise"));
 
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "block", Block);
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "emitSuccess", EmitSuccess);
@@ -123,9 +127,7 @@ Promise::Initialize (v8::Handle<v8::Object> target)
               constructor_template->GetFunction());
 }
 
-v8::Handle<v8::Value>
-Promise::New (const v8::Arguments& args)
-{
+v8::Handle<v8::Value> Promise::New(const v8::Arguments& args) {
   HandleScope scope;
 
   Promise *promise = new Promise();
@@ -135,30 +137,21 @@ Promise::New (const v8::Arguments& args)
   return args.This();
 }
 
-Handle<Value>
-Promise::Block (const Arguments& args)
-{
+Handle<Value> Promise::Block(const Arguments& args) {
   HandleScope scope;
   Promise *promise = ObjectWrap::Unwrap<Promise>(args.Holder());
   promise->Block();
   return Undefined();
 }
 
-v8::Handle<v8::Value>
-Promise::EmitSuccess (const v8::Arguments& args)
-{
+v8::Handle<v8::Value> Promise::EmitSuccess(const v8::Arguments& args) {
   HandleScope scope;
   Promise *promise = ObjectWrap::Unwrap<Promise>(args.Holder());
 
-  int argc = 0;
-  Local<Array> emit_args;
-  if (args[0]->IsArray()) {
-    emit_args = Local<Array>::Cast(args[0]);
-    argc = emit_args->Length();
-  }
+  int argc = args.Length();
   Local<Value> argv[argc];
   for (int i = 0; i < argc; i++) {
-    argv[i] = emit_args->Get(Integer::New(i));
+    argv[i] = args[i];
   }
 
   bool r = promise->EmitSuccess(argc, argv);
@@ -166,21 +159,14 @@ Promise::EmitSuccess (const v8::Arguments& args)
   return r ? True() : False();
 }
 
-v8::Handle<v8::Value>
-Promise::EmitError (const v8::Arguments& args)
-{
+v8::Handle<v8::Value> Promise::EmitError(const v8::Arguments& args) {
   HandleScope scope;
   Promise *promise = ObjectWrap::Unwrap<Promise>(args.Holder());
 
-  int argc = 0;
-  Local<Array> emit_args;
-  if (args[0]->IsArray()) {
-    emit_args = Local<Array>::Cast(args[0]);
-    argc = emit_args->Length();
-  }
+  int argc = args.Length();
   Local<Value> argv[argc];
   for (int i = 0; i < argc; i++) {
-    argv[i] = emit_args->Get(Integer::New(i));
+    argv[i] = args[i];
   }
 
   bool r = promise->EmitError(argc, argv);
@@ -188,32 +174,31 @@ Promise::EmitError (const v8::Arguments& args)
   return r ? True() : False();
 }
 
-void
-Promise::Block (void)
-{
+void Promise::Block(void) {
   blocking_ = true;
 
   assert(prev_ == NULL);
   if (coroutine_top) prev_ = coroutine_top;
   coroutine_top = this;
+  coroutine_stack_size++;
+  if (coroutine_stack_size > 10) {
+    fprintf(stderr, "(node) WARNING: promise.wait() is being called too often.\n");
+  }
 
   ev_loop(EV_DEFAULT_UC_ 0);
 
   assert(!blocking_);
 }
 
-void
-Promise::Destack ()
-{
+void Promise::Destack() {
   assert(coroutine_top == this);
   ev_unloop(EV_DEFAULT_ EVUNLOOP_ONE);
   coroutine_top = prev_;
   prev_ = NULL;
+  coroutine_stack_size--;
 }
 
-void
-Promise::Detach (void)
-{
+void Promise::Detach(void) {
   /* Poor Man's coroutines */
   blocking_ = false;
   while (coroutine_top && !coroutine_top->blocking_) {
@@ -223,9 +208,7 @@ Promise::Detach (void)
   ObjectWrap::Detach();
 }
 
-bool
-Promise::EmitSuccess (int argc, v8::Handle<v8::Value> argv[])
-{
+bool Promise::EmitSuccess(int argc, v8::Handle<v8::Value> argv[]) {
   bool r = Emit("success", argc, argv);
 
   Detach();
@@ -233,9 +216,7 @@ Promise::EmitSuccess (int argc, v8::Handle<v8::Value> argv[])
   return r;
 }
 
-bool
-Promise::EmitError (int argc, v8::Handle<v8::Value> argv[])
-{
+bool Promise::EmitError(int argc, v8::Handle<v8::Value> argv[]) {
   bool r = Emit("error", argc, argv);
 
   Detach();
@@ -243,17 +224,13 @@ Promise::EmitError (int argc, v8::Handle<v8::Value> argv[])
   return r;
 }
 
-Promise*
-Promise::Create (void)
-{
+Promise* Promise::Create(void) {
   HandleScope scope;
 
   Local<Object> handle =
     Promise::constructor_template->GetFunction()->NewInstance();
 
-  Promise *promise = new Promise();
-  promise->Wrap(handle);
-  promise->Attach();
-
-  return promise;
+  return ObjectWrap::Unwrap<Promise>(handle);
 }
+
+}  // namespace node
