@@ -1,7 +1,5 @@
-#include "node.h"
-#include "file.h"
-#include "events.h"
-#include <string.h>
+// Copyright 2009 Ryan Dahl <ry@tinyclouds.org>
+#include <file.h>
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -9,9 +7,11 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <assert.h>
+#include <string.h>
+
+namespace node {
 
 using namespace v8;
-using namespace node;
 
 #define DEV_SYMBOL         String::NewSymbol("dev")
 #define INO_SYMBOL         String::NewSymbol("ino")
@@ -28,40 +28,87 @@ using namespace node;
 #define CTIME_SYMBOL       String::NewSymbol("ctime")
 #define BAD_ARGUMENTS      Exception::TypeError(String::New("Bad argument"))
 
-void
-EIOPromise::Attach (void)
-{
+void EIOPromise::Attach(void) {
   ev_ref(EV_DEFAULT_UC);
   Promise::Attach();
 }
 
-void
-EIOPromise::Detach (void)
-{
+void EIOPromise::Detach(void) {
   Promise::Detach();
   ev_unref(EV_DEFAULT_UC);
 }
 
-EIOPromise*
-EIOPromise::Create (void)
-{
+Handle<Value> EIOPromise::New(const v8::Arguments& args) {
   HandleScope scope;
 
-  Local<Object> handle =
-    Promise::constructor_template->GetFunction()->NewInstance();
-
   EIOPromise *promise = new EIOPromise();
-  promise->Wrap(handle);
+  promise->Wrap(args.This());
 
   promise->Attach();
 
-  return promise;
+  return args.This();
 }
 
-#define NODE_UNIXTIME(t) v8::Date::New(1000*static_cast<double>(t))
-int
-EIOPromise::After (eio_req *req)
-{
+EIOPromise* EIOPromise::Create() {
+  HandleScope scope;
+
+  Local<Object> handle =
+    EIOPromise::constructor_template->GetFunction()->NewInstance();
+
+  return ObjectWrap::Unwrap<EIOPromise>(handle);
+}
+
+static Persistent<FunctionTemplate> stats_constructor_template;
+
+static Local<Object> BuildStatsObject(struct stat * s) {
+  HandleScope scope;
+
+  Local<Object> stats =
+    stats_constructor_template->GetFunction()->NewInstance();
+
+  /* ID of device containing file */
+  stats->Set(DEV_SYMBOL, Integer::New(s->st_dev));
+
+  /* inode number */
+  stats->Set(INO_SYMBOL, Integer::New(s->st_ino));
+
+  /* protection */
+  stats->Set(MODE_SYMBOL, Integer::New(s->st_mode));
+
+  /* number of hard links */
+  stats->Set(NLINK_SYMBOL, Integer::New(s->st_nlink));
+
+  /* user ID of owner */
+  stats->Set(UID_SYMBOL, Integer::New(s->st_uid));
+
+  /* group ID of owner */
+  stats->Set(GID_SYMBOL, Integer::New(s->st_gid));
+
+  /* device ID (if special file) */
+  stats->Set(RDEV_SYMBOL, Integer::New(s->st_rdev));
+
+  /* total size, in bytes */
+  stats->Set(SIZE_SYMBOL, Integer::New(s->st_size));
+
+  /* blocksize for filesystem I/O */
+  stats->Set(BLKSIZE_SYMBOL, Integer::New(s->st_blksize));
+
+  /* number of blocks allocated */
+  stats->Set(BLOCKS_SYMBOL, Integer::New(s->st_blocks));
+
+  /* time of last access */
+  stats->Set(ATIME_SYMBOL, NODE_UNIXTIME_V8(s->st_atime));
+
+  /* time of last modification */
+  stats->Set(MTIME_SYMBOL, NODE_UNIXTIME_V8(s->st_mtime));
+
+  /* time of last status change */
+  stats->Set(CTIME_SYMBOL, NODE_UNIXTIME_V8(s->st_ctime));
+
+  return scope.Close(stats);
+}
+
+int EIOPromise::After(eio_req *req) {
   HandleScope scope;
 
   EIOPromise *promise = reinterpret_cast<EIOPromise*>(req->data);
@@ -87,64 +134,30 @@ EIOPromise::After (eio_req *req)
       break;
 
     case EIO_OPEN:
-    case EIO_WRITE:
       argc = 1;
       argv[0] = Integer::New(req->result);
       break;
 
+    case EIO_WRITE:
+      argc = 1;
+      argv[0] = Integer::New(req->result);
+      assert(req->ptr2);
+      delete [] req->ptr2;
+      break;
+
     case EIO_STAT:
     {
-      Local<Object> stats = Object::New();
       struct stat *s = reinterpret_cast<struct stat*>(req->ptr2);
-      stats->Set(DEV_SYMBOL, Integer::New(s->st_dev)); /* ID of device containing file */
-      stats->Set(INO_SYMBOL, Integer::New(s->st_ino)); /* inode number */
-      stats->Set(MODE_SYMBOL, Integer::New(s->st_mode)); /* protection */
-      stats->Set(NLINK_SYMBOL, Integer::New(s->st_nlink)); /* number of hard links */
-      stats->Set(UID_SYMBOL, Integer::New(s->st_uid)); /* user ID of owner */
-      stats->Set(GID_SYMBOL, Integer::New(s->st_gid)); /* group ID of owner */
-      stats->Set(RDEV_SYMBOL, Integer::New(s->st_rdev)); /* device ID (if special file) */
-      stats->Set(SIZE_SYMBOL, Integer::New(s->st_size)); /* total size, in bytes */
-      stats->Set(BLKSIZE_SYMBOL, Integer::New(s->st_blksize)); /* blocksize for filesystem I/O */
-      stats->Set(BLOCKS_SYMBOL, Integer::New(s->st_blocks)); /* number of blocks allocated */
-      stats->Set(ATIME_SYMBOL, NODE_UNIXTIME(s->st_atime)); /* time of last access */
-      stats->Set(MTIME_SYMBOL, NODE_UNIXTIME(s->st_mtime)); /* time of last modification */
-      stats->Set(CTIME_SYMBOL, NODE_UNIXTIME(s->st_ctime)); /* time of last status change */
       argc = 1;
-      argv[0] = stats;
+      argv[0] = BuildStatsObject(s);
       break;
     }
 
     case EIO_READ:
     {
       argc = 2;
-      // FIXME the following is really ugly!
-      if (promise->encoding_ == RAW) {
-        if (req->result == 0) {
-          argv[0] = Local<Value>::New(Null());
-          argv[1] = Integer::New(0);
-        } else {
-          char *buf = reinterpret_cast<char*>(req->ptr2);
-          size_t len = req->result;
-          Local<Array> array = Array::New(len);
-          for (unsigned int i = 0; i < len; i++) {
-            unsigned char val = reinterpret_cast<const unsigned char*>(buf)[i];
-            array->Set(Integer::New(i), Integer::New(val));
-          }
-          argv[0] = array;
-          argv[1] = Integer::New(req->result);
-        }
-      } else {
-        // UTF8
-        if (req->result == 0) {
-          // eof
-          argv[0] = Local<Value>::New(Null());
-          argv[1] = Integer::New(0);
-        } else {
-          char *buf = reinterpret_cast<char*>(req->ptr2);
-          argv[0] = String::New(buf, req->result);
-          argv[1] = Integer::New(req->result);
-        }
-      }
+      argv[0] = Encode(req->ptr2, req->result, promise->encoding_);
+      argv[1] = Integer::New(req->result);
       break;
     }
 
@@ -181,9 +194,7 @@ EIOPromise::After (eio_req *req)
   return 0;
 }
 
-static Handle<Value>
-Close (const Arguments& args)
-{
+static Handle<Value> Close(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 1 || !args[0]->IsInt32()) {
@@ -195,9 +206,7 @@ Close (const Arguments& args)
   return scope.Close(EIOPromise::Close(fd));
 }
 
-static Handle<Value>
-Stat (const Arguments& args)
-{
+static Handle<Value> Stat(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 1 || !args[0]->IsString()) {
@@ -209,9 +218,7 @@ Stat (const Arguments& args)
   return scope.Close(EIOPromise::Stat(*path));
 }
 
-static Handle<Value>
-Rename (const Arguments& args)
-{
+static Handle<Value> Rename(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsString()) {
@@ -227,8 +234,7 @@ Rename (const Arguments& args)
   return scope.Close(promise->Handle());
 }
 
-static Handle<Value> Unlink (const Arguments& args)
-{
+static Handle<Value> Unlink(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 1 || !args[0]->IsString()) {
@@ -239,9 +245,7 @@ static Handle<Value> Unlink (const Arguments& args)
   return scope.Close(EIOPromise::Unlink(*path));
 }
 
-static Handle<Value>
-RMDir (const Arguments& args)
-{
+static Handle<Value> RMDir(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 1 || !args[0]->IsString()) {
@@ -252,9 +256,7 @@ RMDir (const Arguments& args)
   return scope.Close(EIOPromise::RMDir(*path));
 }
 
-static Handle<Value>
-MKDir (const Arguments& args)
-{
+static Handle<Value> MKDir(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 2 || !args[0]->IsString() || !args[1]->IsInt32()) {
@@ -267,9 +269,7 @@ MKDir (const Arguments& args)
   return scope.Close(EIOPromise::MKDir(*path, mode));
 }
 
-static Handle<Value>
-ReadDir (const Arguments& args)
-{
+static Handle<Value> ReadDir(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 1 || !args[0]->IsString()) {
@@ -280,9 +280,7 @@ ReadDir (const Arguments& args)
   return scope.Close(EIOPromise::ReadDir(*path));
 }
 
-static Handle<Value>
-Open (const Arguments& args)
-{
+static Handle<Value> Open(const Arguments& args) {
   HandleScope scope;
 
   if ( args.Length() < 3
@@ -305,10 +303,9 @@ Open (const Arguments& args)
  * 1 data      the data to write (string = utf8, array = raw)
  * 2 position  if integer, position to write at in the file.
  *             if null, write from the current position
+ * 3 encoding  
  */
-static Handle<Value>
-Write (const Arguments& args)
-{
+static Handle<Value> Write(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 2 || !args[0]->IsInt32()) {
@@ -318,30 +315,17 @@ Write (const Arguments& args)
   int fd = args[0]->Int32Value();
   off_t offset = args[2]->IsNumber() ? args[2]->IntegerValue() : -1;
 
-  char *buf = NULL;
-  size_t len = 0;
-
-  if (args[1]->IsString()) {
-    // utf8 encoding
-    Local<String> string = args[1]->ToString();
-    len = string->Utf8Length();
-    buf = reinterpret_cast<char*>(malloc(len));
-    string->WriteUtf8(buf, len);
-
-  } else if (args[1]->IsArray()) {
-    // raw encoding
-    Local<Array> array = Local<Array>::Cast(args[1]);
-    len = array->Length();
-    buf = reinterpret_cast<char*>(malloc(len));
-    for (unsigned int i = 0; i < len; i++) {
-      Local<Value> int_value = array->Get(Integer::New(i));
-      buf[i] = int_value->Int32Value();
-    }
-
-  } else {
-    return ThrowException(BAD_ARGUMENTS);
+  enum encoding enc = ParseEncoding(args[3]);
+  ssize_t len = DecodeBytes(args[1], enc);
+  if (len < 0) {
+    Local<Value> exception = Exception::TypeError(String::New("Bad argument"));
+    return ThrowException(exception);
   }
 
+  char * buf = new char[len];
+  ssize_t written = DecodeWrite(buf, len, args[1], enc);
+  assert(written == len);
+
   return scope.Close(EIOPromise::Write(fd, buf, len, offset));
 }
 
@@ -352,11 +336,9 @@ Write (const Arguments& args)
  * 1 length    integer. length to read
  * 2 position  if integer, position to read from in the file.
  *             if null, read from the current position
- * 3 encoding  either node.UTF8 or node.RAW
+ * 3 encoding
  */
-static Handle<Value>
-Read (const Arguments& args)
-{
+static Handle<Value> Read(const Arguments& args) {
   HandleScope scope;
 
   if (args.Length() < 2 || !args[0]->IsInt32() || !args[1]->IsNumber()) {
@@ -367,17 +349,14 @@ Read (const Arguments& args)
   size_t len = args[1]->IntegerValue();
   off_t pos = args[2]->IsNumber() ? args[2]->IntegerValue() : -1;
 
-  enum encoding encoding = RAW;
-  if (args[3]->IsInt32()) {
-    encoding = static_cast<enum encoding>(args[3]->Int32Value());
-  }
+  enum encoding encoding = ParseEncoding(args[3]);
 
   return scope.Close(EIOPromise::Read(fd, len, pos, encoding));
 }
 
-void
-File::Initialize (Handle<Object> target)
-{
+Persistent<FunctionTemplate> EIOPromise::constructor_template;
+
+void File::Initialize(Handle<Object> target) {
   HandleScope scope;
 
   NODE_SET_METHOD(target, "close", Close);
@@ -390,4 +369,23 @@ File::Initialize (Handle<Object> target)
   NODE_SET_METHOD(target, "stat", Stat);
   NODE_SET_METHOD(target, "unlink", Unlink);
   NODE_SET_METHOD(target, "write", Write);
+
+  Local<FunctionTemplate> t = FunctionTemplate::New();
+  stats_constructor_template = Persistent<FunctionTemplate>::New(t);
+  target->Set(String::NewSymbol("Stats"),
+      stats_constructor_template->GetFunction());
+
+
+  Local<FunctionTemplate> t2 = FunctionTemplate::New(EIOPromise::New);
+  EIOPromise::constructor_template = Persistent<FunctionTemplate>::New(t2);
+  EIOPromise::constructor_template->Inherit(
+      Promise::constructor_template);
+  EIOPromise::constructor_template->InstanceTemplate()->
+    SetInternalFieldCount(1);
+  EIOPromise::constructor_template->SetClassName(
+      String::NewSymbol("EIOPromise"));
+  target->Set(String::NewSymbol("EIOPromise"),
+      EIOPromise::constructor_template->GetFunction());
 }
+
+}  // end namespace node
