@@ -26,12 +26,14 @@ HTTPConnection::Initialize (Handle<Object> target)
   client_constructor_template = Persistent<FunctionTemplate>::New(t);
   client_constructor_template->Inherit(Connection::constructor_template);
   client_constructor_template->InstanceTemplate()->SetInternalFieldCount(1);
+  client_constructor_template->SetClassName(String::NewSymbol("Client"));
   target->Set(String::NewSymbol("Client"), client_constructor_template->GetFunction());
 
   t = FunctionTemplate::New(NewServer);
   server_constructor_template = Persistent<FunctionTemplate>::New(t);
   server_constructor_template->Inherit(Connection::constructor_template);
   server_constructor_template->InstanceTemplate()->SetInternalFieldCount(1);
+  server_constructor_template->SetClassName(String::NewSymbol("ServerSideConnection"));
 }
 
 Handle<Value>
@@ -69,7 +71,7 @@ HTTPConnection::on_message_begin (http_parser *parser)
 {
   HTTPConnection *connection = static_cast<HTTPConnection*> (parser->data);
   assert(connection->attached_);
-  connection->Emit("message_begin", 0, NULL);
+  connection->Emit("messageBegin", 0, NULL);
   return 0;
 }
 
@@ -78,7 +80,7 @@ HTTPConnection::on_message_complete (http_parser *parser)
 {
   HTTPConnection *connection = static_cast<HTTPConnection*> (parser->data);
   assert(connection->attached_);
-  connection->Emit("message_complete", 0, NULL);
+  connection->Emit("messageComplete", 0, NULL);
   return 0;
 }
 
@@ -94,13 +96,62 @@ HTTPConnection::on_uri (http_parser *parser, const char *buf, size_t len)
 }
 
 int
-HTTPConnection::on_header_field (http_parser *parser, const char *buf, size_t len)
+HTTPConnection::on_query_string (http_parser *parser, const char *buf, size_t len)
 {
   HandleScope scope;
   HTTPConnection *connection = static_cast<HTTPConnection*>(parser->data);
   assert(connection->attached_);
   Local<Value> argv[1] = { String::New(buf, len) };
-  connection->Emit("header_field", 1, argv);
+  connection->Emit("queryString", 1, argv);
+  return 0;
+}
+
+int
+HTTPConnection::on_path (http_parser *parser, const char *buf, size_t len)
+{
+  HandleScope scope;
+  HTTPConnection *connection = static_cast<HTTPConnection*>(parser->data);
+  assert(connection->attached_);
+  Local<Value> argv[1] = { String::New(buf, len) };
+  connection->Emit("path", 1, argv);
+  return 0;
+}
+
+int
+HTTPConnection::on_fragment (http_parser *parser, const char *buf, size_t len)
+{
+  HandleScope scope;
+  HTTPConnection *connection = static_cast<HTTPConnection*>(parser->data);
+  assert(connection->attached_);
+  Local<Value> argv[1] = { String::New(buf, len) };
+  connection->Emit("fragment", 1, argv);
+  return 0;
+}
+
+const static char normalizer[] =
+  "\0------------------------------"
+  "-----------------0123456789-----"
+  "--abcdefghijklmnopqrstuvwxyz----"
+  "--abcdefghijklmnopqrstuvwxyz----"
+  "--------------------------------"
+  "--------------------------------"
+  "--------------------------------"
+  "--------------------------------";
+
+int
+HTTPConnection::on_header_field (http_parser *parser, const char *buf, size_t len)
+{
+  HandleScope scope;
+  HTTPConnection *connection = static_cast<HTTPConnection*>(parser->data);
+  assert(connection->attached_);
+
+  // NORMALIZE
+  size_t i;
+  char *nonconstbuf = (char*)buf; // FIXME
+  for (i = 0; i < len; i++) { nonconstbuf[i] = normalizer[buf[i]]; }
+
+  Local<Value> argv[1] = { String::New(buf, len) };
+  connection->Emit("headerField", 1, argv);
   return 0;
 }
 
@@ -110,8 +161,9 @@ HTTPConnection::on_header_value (http_parser *parser, const char *buf, size_t le
   HandleScope scope;
   HTTPConnection *connection = static_cast<HTTPConnection*>(parser->data);
   assert(connection->attached_);
+
   Local<Value> argv[1] = { String::New(buf, len) };
-  connection->Emit("header_value", 1, argv);
+  connection->Emit("headerValue", 1, argv);
   return 0;
 }
 
@@ -173,7 +225,7 @@ HTTPConnection::on_headers_complete (http_parser *parser)
 
   Local<Value> argv[1] = { message_info };
 
-  connection->Emit("headers_complete", 1, argv);
+  connection->Emit("headerComplete", 1, argv);
 
   return 0;
 }
@@ -187,27 +239,10 @@ HTTPConnection::on_body (http_parser *parser, const char *buf, size_t len)
   assert(connection->attached_);
   HandleScope scope;
 
-  Handle<Value> argv[1];
-
   // TODO each message should have their encoding.
   // don't look at the conneciton for encoding
-
-  if (connection->encoding_ == RAW) {
-    // raw encoding
-    Local<Array> array = Array::New(len);
-    for (size_t i = 0; i < len; i++) {
-      unsigned char val = reinterpret_cast<const unsigned char*>(buf)[i];
-      array->Set(Integer::New(i), Integer::New(val));
-    }
-    argv[0] = array;
-
-  } else {
-    // utf8 or ascii encoding
-    Handle<String> chunk = String::New((const char*)buf, len);
-    argv[0] = chunk;
-  }
-
-  connection->Emit("body", 1, argv);
+  Local<Value> data = Encode(buf, len, connection->encoding_);
+  connection->Emit("body", 1, &data);
 
   return 0;
 }
@@ -223,6 +258,7 @@ HTTPServer::Initialize (Handle<Object> target)
   constructor_template = Persistent<FunctionTemplate>::New(t);
   constructor_template->Inherit(Server::constructor_template);
   constructor_template->InstanceTemplate()->SetInternalFieldCount(1);
+  constructor_template->SetClassName(String::NewSymbol("Server"));
   target->Set(String::NewSymbol("Server"), constructor_template->GetFunction());
 }
 
