@@ -24,6 +24,9 @@ protected:
     http_parser_init (&parser_, type);
     parser_.on_message_begin    = on_message_begin;
     parser_.on_uri              = on_uri;
+    parser_.on_path             = on_path;
+    parser_.on_fragment         = on_fragment;
+    parser_.on_query_string     = on_query_string;
     parser_.on_header_field     = on_header_field;
     parser_.on_header_value     = on_header_value;
     parser_.on_headers_complete = on_headers_complete;
@@ -36,6 +39,9 @@ protected:
 
   static int on_message_begin (http_parser *parser);
   static int on_uri (http_parser *parser, const char *at, size_t length);
+  static int on_query_string (http_parser *parser, const char *at, size_t length);
+  static int on_path (http_parser *parser, const char *at, size_t length);
+  static int on_fragment (http_parser *parser, const char *at, size_t length);
   static int on_header_field (http_parser *parser, const char *buf, size_t len);
   static int on_header_value (http_parser *parser, const char *buf, size_t len);
   static int on_headers_complete (http_parser *parser);
