@@ -1,7 +1,5 @@
-#include "net.h"
-#include "events.h"
-
-#include <udns.h>
+// Copyright 2009 Ryan Dahl <ry@tinyclouds.org>
+#include <net.h>
 
 #include <assert.h>
 #include <stdlib.h>
@@ -15,10 +13,11 @@
 #include <netinet/in.h> /* sockaddr_in, sockaddr_in6 */
 
 using namespace v8;
-using namespace node;
+
+namespace node {
 
 #define UTF8_SYMBOL           String::NewSymbol("utf8")
-#define RAW_SYMBOL            String::NewSymbol("raw")
+#define BINARY_SYMBOL         String::NewSymbol("binary")
 #define ASCII_SYMBOL          String::NewSymbol("ascii")
 
 #define SERVER_SYMBOL         String::NewSymbol("server")
@@ -48,9 +47,7 @@ static const struct addrinfo client_tcp_hints =
 
 Persistent<FunctionTemplate> Connection::constructor_template;
 
-void
-Connection::Initialize (v8::Handle<v8::Object> target)
-{
+void Connection::Initialize(v8::Handle<v8::Object> target) {
   HandleScope scope;
 
   Local<FunctionTemplate> t = FunctionTemplate::New(New);
@@ -58,6 +55,7 @@ Connection::Initialize (v8::Handle<v8::Object> target)
   constructor_template = Persistent<FunctionTemplate>::New(t);
   constructor_template->Inherit(EventEmitter::constructor_template);
   constructor_template->InstanceTemplate()->SetInternalFieldCount(1);
+  constructor_template->SetClassName(String::NewSymbol("Connection"));
 
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "connect", Connect);
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "send", Send);
@@ -67,17 +65,18 @@ Connection::Initialize (v8::Handle<v8::Object> target)
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "readPause", ReadPause);
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "readResume", ReadResume);
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "setTimeout", SetTimeout);
+  NODE_SET_PROTOTYPE_METHOD(constructor_template, "setNoDelay", SetNoDelay);
 
   constructor_template->PrototypeTemplate()->SetAccessor(
       READY_STATE_SYMBOL,
       ReadyStateGetter);
 
-  target->Set(String::NewSymbol("Connection"), constructor_template->GetFunction());
+  target->Set(String::NewSymbol("Connection"),
+      constructor_template->GetFunction());
 }
 
-Handle<Value>
-Connection::ReadyStateGetter (Local<String> property, const AccessorInfo& info)
-{
+Handle<Value> Connection::ReadyStateGetter(Local<String> property,
+                                           const AccessorInfo& info) {
   Connection *connection = ObjectWrap::Unwrap<Connection>(info.This());
   assert(connection);
 
@@ -98,12 +97,11 @@ Connection::ReadyStateGetter (Local<String> property, const AccessorInfo& info)
   }
 
   assert(0 && "This shouldnt happen");
-  return ThrowException(String::New("This shouldn't happen."));
+  return ThrowException(Exception::Error(
+        String::New("This shouldn't happen.")));
 }
 
-void
-Connection::Init (void)
-{
+void Connection::Init() {
   resolving_ = false;
   evcom_stream_init(&stream_);
   stream_.on_connect = Connection::on_connect;
@@ -113,15 +111,12 @@ Connection::Init (void)
   stream_.data = this;
 }
 
-Connection::~Connection ()
-{
+Connection::~Connection() {
   assert(stream_.recvfd < 0 && "garbage collecting open Connection");
   assert(stream_.sendfd < 0 && "garbage collecting open Connection");
 }
 
-Handle<Value>
-Connection::New (const Arguments& args)
-{
+Handle<Value> Connection::New(const Arguments& args) {
   HandleScope scope;
 
   Connection *connection = new Connection();
@@ -130,9 +125,7 @@ Connection::New (const Arguments& args)
   return args.This();
 }
 
-Handle<Value>
-Connection::Connect (const Arguments& args)
-{
+Handle<Value> Connection::Connect(const Arguments& args) {
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.Holder());
 
   assert(connection);
@@ -140,12 +133,13 @@ Connection::Connect (const Arguments& args)
   HandleScope scope;
 
   if (connection->ReadyState() == EVCOM_CLOSED) {
-    connection->Init(); // in case we're reusing the socket
+    connection->Init();  // in case we're reusing the socket
     assert(connection->ReadyState() == EVCOM_INITIALIZED);
   }
 
   if (connection->ReadyState() != EVCOM_INITIALIZED) {
-    Local<Value> exception = Exception::Error(String::New("Socket is not in CLOSED state."));
+    Local<Value> exception = Exception::Error(
+        String::New("Socket is not in CLOSED state."));
     return ThrowException(exception);
   }
 
@@ -153,7 +147,8 @@ Connection::Connect (const Arguments& args)
   assert(connection->stream_.sendfd < 0);
 
   if (args.Length() == 0) {
-    Local<Value> exception = Exception::TypeError(String::New("First argument must be a port number"));
+    Local<Value> exception = Exception::TypeError(
+        String::New("First argument must be a port number"));
     return ThrowException(exception);
   }
 
@@ -178,18 +173,13 @@ Connection::Connect (const Arguments& args)
    * In the future I will move to a system using adns or udns:
    * http://lists.schmorp.de/pipermail/libev/2009q1/000632.html
    */
-  eio_custom( Connection::Resolve
-            , EIO_PRI_DEFAULT
-            , Connection::AfterResolve
-            , connection
-            );
+  eio_custom(Connection::Resolve, EIO_PRI_DEFAULT, Connection::AfterResolve,
+      connection);
 
   return Undefined();
 }
 
-int
-Connection::Resolve (eio_req *req)
-{
+int Connection::Resolve(eio_req *req) {
   Connection *connection = static_cast<Connection*> (req->data);
   struct addrinfo *address = NULL;
 
@@ -209,9 +199,7 @@ Connection::Resolve (eio_req *req)
   return 0;
 }
 
-static struct addrinfo *
-AddressDefaultToIPv4 (struct addrinfo *address_list)
-{
+static struct addrinfo * AddressDefaultToIPv4(struct addrinfo *address_list) {
   struct addrinfo *address = NULL;
 
   for (address = address_list; address != NULL; address = address->ai_next) {
@@ -221,9 +209,7 @@ AddressDefaultToIPv4 (struct addrinfo *address_list)
   return address == NULL ? address_list : address;
 }
 
-int
-Connection::AfterResolve (eio_req *req)
-{
+int Connection::AfterResolve(eio_req *req) {
   ev_unref(EV_DEFAULT_UC);
 
   Connection *connection = static_cast<Connection*> (req->data);
@@ -245,7 +231,7 @@ Connection::AfterResolve (eio_req *req)
 
   // no error. return.
   if (req->result == 0) {
-    evcom_stream_attach (EV_DEFAULT_UC_ &connection->stream_);
+    evcom_stream_attach(EV_DEFAULT_UC_ &connection->stream_);
     goto out;
   }
 
@@ -261,21 +247,19 @@ Connection::AfterResolve (eio_req *req)
 
   connection->Detach();
 
-out:
+ out:
   return 0;
 }
 
-Handle<Value>
-Connection::SetEncoding (const Arguments& args)
-{
+Handle<Value> Connection::SetEncoding(const Arguments& args) {
   HandleScope scope;
 
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.This());
   assert(connection);
 
   if (!args[0]->IsString()) {
-    connection->encoding_ = RAW;
-    return scope.Close(RAW_SYMBOL);
+    connection->encoding_ = BINARY;
+    return scope.Close(BINARY_SYMBOL);
   }
 
   switch (ParseEncoding(args[0])) {
@@ -287,16 +271,16 @@ Connection::SetEncoding (const Arguments& args)
       connection->encoding_ = UTF8;
       return scope.Close(UTF8_SYMBOL);
 
-    case RAW:
-    default:
-      connection->encoding_ = RAW;
-      return scope.Close(RAW_SYMBOL);
+    case BINARY:
+      connection->encoding_ = BINARY;
+      return scope.Close(BINARY_SYMBOL);
   }
+  assert(0 && "this shouldn't happen");
+  return ThrowException(Exception::Error(
+        String::New("Could not parse encoding. This is a Node bug.")));
 }
 
-Handle<Value>
-Connection::ReadPause (const Arguments& args)
-{
+Handle<Value> Connection::ReadPause(const Arguments& args) {
   HandleScope scope;
 
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.This());
@@ -307,9 +291,7 @@ Connection::ReadPause (const Arguments& args)
   return Undefined();
 }
 
-Handle<Value>
-Connection::ReadResume (const Arguments& args)
-{
+Handle<Value> Connection::ReadResume(const Arguments& args) {
   HandleScope scope;
 
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.This());
@@ -320,24 +302,20 @@ Connection::ReadResume (const Arguments& args)
   return Undefined();
 }
 
-Handle<Value>
-Connection::SetTimeout (const Arguments& args)
-{
+Handle<Value> Connection::SetTimeout(const Arguments& args) {
   HandleScope scope;
 
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.This());
   assert(connection);
 
-  float timeout = (float)(args[0]->IntegerValue()) / 1000;
+  float timeout = NODE_V8_UNIXTIME(args[0]);
 
   connection->SetTimeout(timeout);
 
   return Undefined();
 }
 
-Handle<Value>
-Connection::Close (const Arguments& args)
-{
+Handle<Value> Connection::Close(const Arguments& args) {
   HandleScope scope;
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.Holder());
   assert(connection);
@@ -346,9 +324,7 @@ Connection::Close (const Arguments& args)
   return Undefined();
 }
 
-Handle<Value>
-Connection::ForceClose (const Arguments& args)
-{
+Handle<Value> Connection::ForceClose(const Arguments& args) {
   HandleScope scope;
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.Holder());
   assert(connection);
@@ -358,101 +334,56 @@ Connection::ForceClose (const Arguments& args)
   return Undefined();
 }
 
-
-Handle<Value>
-Connection::Send (const Arguments& args)
-{
+Handle<Value> Connection::SetNoDelay(const Arguments& args) {
   HandleScope scope;
   Connection *connection = ObjectWrap::Unwrap<Connection>(args.Holder());
-  assert(connection);
 
-  if ( connection->ReadyState() != EVCOM_CONNECTED_RW
-    && connection->ReadyState() != EVCOM_CONNECTED_WO
-     )
-  {
-    Local<Value> exception = Exception::Error(String::New("Socket is not open for writing"));
-    return ThrowException(exception);
+  bool no_delay = true;
+  if (args.Length() > 0) {
+    no_delay = args[0]->IsTrue();
   }
 
-  // XXX
-  // A lot of improvement can be made here. First of all we're allocating
-  // evcom_bufs for every send which is clearly inefficent - it should use a
-  // memory pool or ring buffer. Of course, expressing binary data as an
-  // array of integers is extremely inefficent. This can improved when v8
-  // bug 270 (http://code.google.com/p/v8/issues/detail?id=270) has been
-  // addressed.
-
-  if (args[0]->IsString()) {
-    enum encoding enc = ParseEncoding(args[1]);
-    Local<String> s = args[0]->ToString();
-    size_t len = s->Utf8Length();
-    char buf[len];
-    switch (enc) {
-      case RAW:
-      case ASCII:
-        s->WriteAscii(buf, 0, len);
-        break;
-
-      case UTF8:
-        s->WriteUtf8(buf, len);
-        break;
-
-      default:
-        assert(0 && "unhandled string encoding");
-    }
-    connection->Send(buf, len);
-
-  } else if (args[0]->IsArray()) {
-    Handle<Array> array = Handle<Array>::Cast(args[0]);
-    size_t len = array->Length();
-    char buf[len];
-    for (size_t i = 0; i < len; i++) {
-      Local<Value> int_value = array->Get(Integer::New(i));
-      buf[i] = int_value->IntegerValue();
-    }
-    connection->Send(buf, len);
-
-  } else {
-    Local<Value> exception = Exception::TypeError(String::New("Bad argument"));
-    return ThrowException(exception);
-  }
+  connection->SetNoDelay(no_delay);
 
   return Undefined();
 }
 
-void
-Connection::OnReceive (const void *buf, size_t len)
-{
+Handle<Value> Connection::Send(const Arguments& args) {
   HandleScope scope;
+  Connection *connection = ObjectWrap::Unwrap<Connection>(args.Holder());
+  assert(connection);
 
-  const int argc = 1;
-  Handle<Value> argv[argc];
-
-  if(len) {
-    if (encoding_ == RAW) {
-      // raw encoding
-      Local<Array> array = Array::New(len);
-      for (size_t i = 0; i < len; i++) {
-        unsigned char val = static_cast<const unsigned char*>(buf)[i];
-        array->Set(Integer::New(i), Integer::New(val));
-      }
-      argv[0] = array;
-
-    } else {
-      // utf8 or ascii encoding
-      Handle<String> chunk = String::New((const char*)buf, len);
-      argv[0] = chunk;
-    }
-  } else {
-    argv[0] = Local<Value>::New(Null());
+  if (connection->ReadyState() != EVCOM_CONNECTED_RW &&
+      connection->ReadyState() != EVCOM_CONNECTED_WO) {
+    Local<Value> exception = Exception::Error(
+        String::New("Socket is not open for writing"));
+    return ThrowException(exception);
   }
 
-  Emit("receive", argc, argv);
+  enum encoding enc = ParseEncoding(args[1]);
+  ssize_t len = DecodeBytes(args[0], enc);
+
+  if (len < 0) {
+    Local<Value> exception = Exception::TypeError(String::New("Bad argument"));
+    return ThrowException(exception);
+  }
+
+  char * buf = new char[len];
+  ssize_t written = DecodeWrite(buf, len, args[0], enc);
+  assert(written == len);
+  connection->Send(buf, written);
+  delete [] buf;
+
+  return scope.Close(Integer::New(written));
 }
 
-void
-Connection::OnClose ()
-{
+void Connection::OnReceive(const void *buf, size_t len) {
+  HandleScope scope;
+  Local<Value> data = Encode(buf, len, encoding_);
+  Emit("receive", 1, &data);
+}
+
+void Connection::OnClose() {
   HandleScope scope;
 
   Handle<Value> argv[1];
@@ -461,28 +392,38 @@ Connection::OnClose ()
   Emit("close", 1, argv);
 }
 
-#define DEFINE_SIMPLE_CALLBACK(name, type)                          \
-void name ()                                                        \
-{                                                                   \
-  HandleScope scope;                                                \
-  Emit (type, 0, NULL);                                             \
+void Connection::OnConnect() {
+  HandleScope scope;
+
+  if (stream_.server) {
+    Server *server = static_cast<Server*>(stream_.server->data);
+    Local<Value> value = Local<Value>::New(handle_);
+    server->Emit("connection", 1, &value);
+  }
+
+  Emit("connect", 0, NULL);
 }
 
-DEFINE_SIMPLE_CALLBACK(Connection::OnConnect, "connect")
-DEFINE_SIMPLE_CALLBACK(Connection::OnTimeout, "timeout")
-DEFINE_SIMPLE_CALLBACK(Connection::OnEOF, "eof")
+void Connection::OnTimeout() {
+  HandleScope scope;
+  Emit("timeout", 0, NULL);
+}
+
+void Connection::OnEOF() {
+  HandleScope scope;
+  Emit("eof", 0, NULL);
+}
 
 Persistent<FunctionTemplate> Server::constructor_template;
 
-void
-Server::Initialize (Handle<Object> target)
-{
+void Server::Initialize(Handle<Object> target) {
   HandleScope scope;
 
   Local<FunctionTemplate> t = FunctionTemplate::New(New);
   constructor_template = Persistent<FunctionTemplate>::New(t);
   constructor_template->Inherit(EventEmitter::constructor_template);
   constructor_template->InstanceTemplate()->SetInternalFieldCount(1);
+  constructor_template->SetClassName(String::NewSymbol("Server"));
 
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "listen", Listen);
   NODE_SET_PROTOTYPE_METHOD(constructor_template, "close", Close);
@@ -490,10 +431,7 @@ Server::Initialize (Handle<Object> target)
   target->Set(String::NewSymbol("Server"), constructor_template->GetFunction());
 }
 
-
-static Local<String>
-GetAddressString (struct sockaddr *addr)
-{
+static Local<String> GetAddressString(struct sockaddr *addr) {
   HandleScope scope;
   char ip[INET6_ADDRSTRLEN];
   Local<String> remote_address;
@@ -508,27 +446,23 @@ GetAddressString (struct sockaddr *addr)
     inet_ntop(AF_INET6, &(sa6->sin6_addr), ip, INET6_ADDRSTRLEN);
     remote_address = String::New(ip);
 
-  } else assert(0 && "received a bad sa_family");
+  } else {
+    assert(0 && "received a bad sa_family");
+  }
 
   return scope.Close(remote_address);
 }
 
-Handle<FunctionTemplate>
-Server::GetConnectionTemplate (void)
-{
+Handle<FunctionTemplate> Server::GetConnectionTemplate() {
   return Connection::constructor_template;
 }
 
-Connection*
-Server::UnwrapConnection (Local<Object> connection)
-{
+Connection* Server::UnwrapConnection(Local<Object> connection) {
   HandleScope scope;
   return ObjectWrap::Unwrap<Connection>(connection);
 }
 
-Connection*
-Server::OnConnection (struct sockaddr *addr)
-{
+Connection* Server::OnConnection(struct sockaddr *addr) {
   HandleScope scope;
 
   TryCatch try_catch;
@@ -548,18 +482,12 @@ Server::OnConnection (struct sockaddr *addr)
   Connection *connection = UnwrapConnection(js_connection);
   if (!connection) return NULL;
 
-  Handle<Value> argv[1] = { js_connection };
-
-  Emit("connection", 1, argv);
-
   connection->Attach();
 
   return connection;
 }
 
-void
-Server::OnClose (int errorno)
-{
+void Server::OnClose(int errorno) {
   HandleScope scope;
 
   Handle<Value> argv[1] = { Integer::New(errorno) };
@@ -567,12 +495,7 @@ Server::OnClose (int errorno)
   Emit("close", 1, argv);
 }
 
-// TODO Server->SetOptions
-// TODO Server -> Server rename
-
-Handle<Value>
-Server::New (const Arguments& args)
-{
+Handle<Value> Server::New(const Arguments& args) {
   HandleScope scope;
 
   Server *server = new Server();
@@ -581,23 +504,26 @@ Server::New (const Arguments& args)
   return args.This();
 }
 
-Handle<Value>
-Server::Listen (const Arguments& args)
-{
+Handle<Value> Server::Listen(const Arguments& args) {
   Server *server = ObjectWrap::Unwrap<Server>(args.Holder());
   assert(server);
 
   HandleScope scope;
 
   if (args.Length() == 0) {
-    Local<Value> exception = Exception::TypeError(String::New("First argument must be a port number"));
+    Local<Value> exception = Exception::TypeError(
+        String::New("First argument must be a port number"));
     return ThrowException(exception);
   }
 
   String::AsciiValue port(args[0]->ToString());
 
+#ifndef DNS_MAXNAME
+# define DNS_MAXNAME 1024
+#endif
+
   char host[DNS_MAXNAME+1] = "\0";
-  int backlog = 1024;
+  int backlog = 128;
 
   if (args.Length() == 2) {
     if (args[1]->IsInt32()) {
@@ -621,9 +547,12 @@ Server::Listen (const Arguments& args)
   // For servers call getaddrinfo inline. This is blocking but it shouldn't
   // matter much. If someone actually complains then simply swap it out
   // with a libeio call.
-  struct addrinfo *address = NULL,
-                  *address_list = NULL;
-  int r = getaddrinfo(strlen(host) ? host : NULL, *port, &server_tcp_hints, &address_list);
+  struct addrinfo * address = NULL,
+                  * address_list = NULL;
+
+  int r = getaddrinfo(strlen(host) ?
+      host : NULL, *port, &server_tcp_hints, &address_list);
+
   if (r != 0) {
     Local<Value> exception = Exception::Error(String::New(strerror(errno)));
     return ThrowException(exception);
@@ -638,12 +567,12 @@ Server::Listen (const Arguments& args)
   return Undefined();
 }
 
-Handle<Value>
-Server::Close (const Arguments& args)
-{
+Handle<Value> Server::Close(const Arguments& args) {
   Server *server = ObjectWrap::Unwrap<Server>(args.Holder());
   assert(server);
 
   server->Close();
   return Undefined();
 }
+
+}  // namespace node
