@@ -1,4 +1,6 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
+tcp = require("/tcp.js");
+http = require("/http.js");
 
 // Make sure no exceptions are thrown when receiving malformed HTTP
 // requests.
@@ -7,7 +9,7 @@ port = 9999;
 nrequests_completed = 0;
 nrequests_expected = 1;
 
-var s = node.http.createServer(function (req, res) {
+var s = http.createServer(function (req, res) {
   puts("req: " + JSON.stringify(req.uri));
 
   res.sendHeader(200, {"Content-Type": "text/plain"});
@@ -18,7 +20,7 @@ var s = node.http.createServer(function (req, res) {
 });
 s.listen(port);
 
-var c = node.tcp.createConnection(port);
+var c = tcp.createConnection(port);
 c.addListener("connect", function () {
   c.send("GET /hello?foo=%99bar HTTP/1.1\r\n\r\n");
   c.close();
