@@ -1,20 +1,21 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
+http = require("/http.js");
 
 var PROXY_PORT = 8869;
 var BACKEND_PORT = 8870;
 
-var backend = node.http.createServer(function (req, res) {
-  // node.debug("backend");
+var backend = http.createServer(function (req, res) {
+  // debug("backend");
   res.sendHeader(200, {"content-type": "text/plain"});
   res.sendBody("hello world\n");
   res.finish();
 });
-// node.debug("listen backend")
+// debug("listen backend")
 backend.listen(BACKEND_PORT);
 
-var proxy_client = node.http.createClient(BACKEND_PORT);
-var proxy = node.http.createServer(function (req, res) {
-  node.debug("proxy req headers: " + JSON.stringify(req.headers));
+var proxy_client = http.createClient(BACKEND_PORT);
+var proxy = http.createServer(function (req, res) {
+  debug("proxy req headers: " + JSON.stringify(req.headers));
   var proxy_req = proxy_client.get(req.uri.path);
   proxy_req.finish(function(proxy_res) {
     res.sendHeader(proxy_res.statusCode, proxy_res.headers);
@@ -23,27 +24,27 @@ var proxy = node.http.createServer(function (req, res) {
     });
     proxy_res.addListener("complete", function() {
       res.finish();
-      // node.debug("proxy res");
+      // debug("proxy res");
     });
   });
 });
-// node.debug("listen proxy")
+// debug("listen proxy")
 proxy.listen(PROXY_PORT);
 
 var body = "";
 
-var client = node.http.createClient(PROXY_PORT);
+var client = http.createClient(PROXY_PORT);
 var req = client.get("/test");
-// node.debug("client req")
+// debug("client req")
 req.finish(function (res) {
-  // node.debug("got res");
+  // debug("got res");
   assertEquals(200, res.statusCode);
   res.setBodyEncoding("utf8");
   res.addListener("body", function (chunk) { body += chunk; });
   res.addListener("complete", function () {
     proxy.close();
     backend.close();
-    // node.debug("closed both");
+    // debug("closed both");
   });
 });
 
