@@ -1,4 +1,6 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
+tcp = require("/tcp.js");
+http = require("/http.js");
 
 var port = 8222;
 
@@ -7,13 +9,15 @@ var requests_sent = 0;
 var server_response = "";
 var client_got_eof = false;
 
-node.http.createServer(function (req, res) {
+http.createServer(function (req, res) {
   res.id = request_number;
   req.id = request_number++;
 
   if (req.id == 0) {
     assertEquals("GET", req.method);
     assertEquals("/hello", req.uri.path);
+    assertEquals("world", req.uri.params["hello"]);
+    assertEquals("b==ar", req.uri.params["foo"]);
   }
 
   if (req.id == 1) {
@@ -31,12 +35,12 @@ node.http.createServer(function (req, res) {
 
 }).listen(port);
 
-var c = node.tcp.createConnection(port);
+var c = tcp.createConnection(port);
 
 c.setEncoding("utf8");
 
 c.addListener("connect", function () {
-  c.send( "GET /hello HTTP/1.1\r\n\r\n" );
+  c.send( "GET /hello?hello=world&foo=b==ar HTTP/1.1\r\n\r\n" );
   requests_sent += 1;
 });
 
