@@ -1,4 +1,4 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
 
 var dirname = node.path.dirname(__filename);
 var fixtures = node.path.join(dirname, "fixtures");
