@@ -1,4 +1,7 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
+
+debug("load test-module-loading.js");
+
 var a = require("fixtures/a.js");
 var d = require("fixtures/b/d.js");
 var d2 = require("fixtures/b/d.js");
@@ -35,4 +38,6 @@ process.addListener("exit", function () {
 
   assertInstanceof(d2.D, Function);
   assertEquals("D done", d2.D());
+
+  puts("exit");
 });
