@@ -1,4 +1,4 @@
-include("mjsunit.js");
+node.mixin(require("common.js"));
 
 function timer (t) {
   var promise = new node.Promise();
